# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ImprovedMoinSearch library

    @copyright: 2010 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.search import searchPages
from MoinMoin.Page import Page
from MoinMoin import wikiutil
import re

heading_regexp = re.compile(r"^(?P<level>=+)(?P<heading>.*?)(?P=level)$", re.UNICODE | re.MULTILINE)
paragraph_regexp = re.compile(r"(?P<paragraph>(?:^[^#=\s].*$\n)+)", re.UNICODE | re.MULTILINE)

def range_groups(min_name, max_name):
    return r"(?P<%s>\d+)?(?:\s*-\s*(?P<%s>\d+))?" % (min_name, max_name)

format_options_regexp = re.compile(r"("
    "(?P<heading>(heading|title|h)\s*" + range_groups("min_heading", "max_heading") + ")"
    "|(?P<paragraph>(paragraph|para|p)\s*(?P<paragraph_number>\d+)?)"
    ")", re.UNICODE)

def getSearchResultPages(request, query, **kw):

    """
    Return matching pages using the given 'request' and search 'query'. Optional
    keyword arguments are passed to the underlying search infrastructure.
    """

    results = searchPages(request, query, **kw)
    return results.hits

def getFirstPageHeading(request, page, min_level=None, max_level=None):

    """
    Using the given 'request', return the first heading in the given 'page'
    having a heading level of at least 'min_level' (which is undefined if not
    specified) and at most 'max_level' (which is undefined if not specified).
    """

    full_page = Page(request, page.page_name)
    body = full_page.get_raw_body()

    for match in heading_regexp.finditer(body):
        level = len(match.group("level"))

        if (min_level is None or level >= min_level) and \
            (max_level is None or level <= max_level):

            return match.group("heading")

    return None

def getParagraph(request, page, number=None):

    full_page = Page(request, page.page_name)
    body = full_page.get_raw_body()

    for i, match in enumerate(paragraph_regexp.finditer(body)):
        if number is None or i == number:
            return match.group("paragraph")

    return None

def getPageName(request, page):
    return page.page_name

def formatResultPages(request, formatter, pages, paging, format, page_from=0):

    """
    Using the given 'request' and 'formatter', return a formatted string showing
    the result 'pages', providing paging controls when 'paging' is set to a true
    value, and providing page details according to the given 'format'.

    If the optional 'pages_from' parameter is set, the result pages from the
    given result (specified within a range from 0 to the length of the 'pages'
    collection) will be shown.
    """

    actions = []

    if format:
        for match in format_options_regexp.finditer(format):
            if match.group("heading"):
                actions.append((getFirstPageHeading, map(int_or_none, (match.group("min_heading"), match.group("max_heading")))))
            elif match.group("paragraph"):
                actions.append((getParagraph, map(int_or_none, (match.group("paragraph_number"),))))
    else:
        actions.append((getPageName, ()))

    # Use paging only when there are enough results.

    results_per_page = request.cfg.search_results_per_page
    paging = paging and len(pages) > results_per_page

    if paging:
        pages_to_show = pages[page_from:page_from + results_per_page]
    else:
        pages_to_show = pages

    # Prepare the output.

    output = []
    output.append(formatter.number_list(on=1, start=page_from + 1))

    for page in pages_to_show:
        output.append(formatter.listitem(on=1))

        first = 1
        for action, args in actions:
            if first:
                output.append(formatter.pagelink(on=1, pagename=page.page_name))
            else:
                output.append(" ")

            text = action(request, page, *args)
            output.append(formatter.text(text))

            if first:
                output.append(formatter.pagelink(on=0))
            first = 0

        output.append(formatter.listitem(on=0))

    output.append(formatter.number_list(on=0))

    # Show paging navigation.

    if paging:
        output.append(formatPagingNavigation(request, formatter, pages, page_from))

    return "".join(output)

def formatPagingNavigation(request, formatter, pages, page_from=0):

    """
    Using the given 'request' and 'formatter', return a formatted string showing
    the paging navigation for the result 'pages', according to the 'page_from'
    indicator which provides the current position in the result set.
    """

    page = formatter.page
    pagename = page.page_name
    _ = request.getText

    output = []

    results_per_page = request.cfg.search_results_per_page
    number_of_results = len(pages)

    pages_total = number_of_results / results_per_page
    pages_before = page_from / results_per_page
    pages_after = ((number_of_results - page_from) / results_per_page) - 1

    querydict = wikiutil.parseQueryString(request.query_string)

    output.append(formatter.paragraph(on=1))
    output.append(formatter.text(_("Result pages:")))
    output.append(formatter.text(" "))

    n = 0
    while n < pages_before:
        output.append(formatter.pagelink(on=1, pagename=pagename, querystr=getPagingQueryString(querydict, n * results_per_page)))
        output.append(formatter.text(str(n + 1)))
        output.append(formatter.pagelink(on=0))
        output.append(formatter.text(" "))
        n += 1

    output.append(formatter.text(str(n + 1)))
    output.append(formatter.text(" "))
    n += 1

    while n <= pages_total:
        output.append(formatter.pagelink(on=1, pagename=pagename, querystr=getPagingQueryString(querydict, n * results_per_page)))
        output.append(formatter.text(str(n + 1)))
        output.append(formatter.pagelink(on=0))
        output.append(formatter.text(" "))
        n += 1

    output.append(formatter.paragraph(on=0))

    return "".join(output)

def getPagingQueryString(querydict, page_from):
    querydict["from"] = page_from
    return wikiutil.makeQueryString(querydict)

def int_or_none(x):
    if x is None:
        return x
    else:
        return int(x)

# vim: tabstop=4 expandtab shiftwidth=4
